// -----------------------------------------
// Include

// Circle fitting class lib
#include "CircleFittingClass.h"
#include "CircleFittingClass_ErrorCodes.h"

// Standard libs
#include <math.h>
#include <stdlib.h>

// -----------------------------------------
// Constructor
CircleFittingClass::CircleFittingClass(void)
{
}

// -----------------------------------------
// Destructort
CircleFittingClass::~CircleFittingClass(void)
{
}

// -----------------------------------------
// Reset points
void CircleFittingClass::mResetPoints()
{
	this->_mSetNPoints(0);
}

// -----------------------------------------
// Get number of points
int CircleFittingClass::_mGetNPoints()
{
	return this->_vNPoints;
}

// -----------------------------------------
// Set number of points
int CircleFittingClass::_mSetNPoints(int NPoints)
{
	// -----------------------------------------
	// Variables

	int Err=0;		// Error code

	// -----------------------------------------
	// Set NPoints

	// Verify that the number of points does not exceed the max
	if ((NPoints<0)|(NPoints>_CLASSCIRCLEFITTING_MAXNPOINTS))
		return (Err=_CLASSCIRCLEFITTING_ERRORCANNOTSETNPOINTS);

	// Set NPoints
	this->_vNPoints=NPoints;

	// -----------------------------------------
	// Return

	return Err;
}

// -----------------------------------------
// Add point
int	CircleFittingClass::mAddPoint(double X, double Y)
{
	// -----------------------------------------
	// Variables

	int Err=0;		// Error code
	int	nPoint;		// Point index

	// -----------------------------------------
	// Add point

	// Get point index
	nPoint=this->_mGetNPoints();

	// Verify current number of point
	if (nPoint==_CLASSCIRCLEFITTING_MAXNPOINTS)
		return (_CLASSCIRCLEFITTING_ERRORCANNOTADDPOINT);

	// Add point
	this->_vPoints[nPoint][0]=X;
	this->_vPoints[nPoint][1]=Y;

	// Adj number of point
	this->_mSetNPoints(nPoint+1);

	// -----------------------------------------
	// Return

	return Err;
}

// -----------------------------------------
// Compute norm
double	CircleFittingClass::_mComputeNorm(double X0, double Y0, double X1, double Y1)
{
	return sqrt((X1-X0)*(X1-X0)+(Y1-Y0)*(Y1-Y0));
}

// -----------------------------------------
// Cluster baricenter circle fitting
int	CircleFittingClass::mClusterBaricenterCircleFitting()
{
	// -----------------------------------------
	// Variables

	int		Err=0;		// Error code
	int		nPoint;		// Point index
	
	// -----------------------------------------
	// Cluster baricenter circle fitting

	// Compute baricenter
	if ((Err=this->_mComputeBaricenter())>0)
		return Err;

	// Compute radius
	this->_vRadius=0;
	for (nPoint=0; nPoint<this->_mGetNPoints(); nPoint++)
		{
		  this->_vRadius=this->_mComputeNorm(this->_vPoints[nPoint][0],
						    				  this->_vPoints[nPoint][1],
										      this->_vCenterX,
									 	      this->_vCenterY);
		  if (this->_vRadius<0)
		    this->_vRadius=0;
		}
	
	// -----------------------------------------
	// Return

	return Err;
}

// -----------------------------------------
// Edge baricenter circle fitting
int	CircleFittingClass::mEdgeBaricenterCircleFitting()
{
	// -----------------------------------------
	// Variables

	int		Err=0;		// Error code
	double	SumRadius;	// SumRadius
	int		nPoint;		// Point index

	// -----------------------------------------
	// Edge baricenter circle fitting

	// Compute baricenter
	if ((Err=this->_mComputeBaricenter())>0)
		return Err;

	// Compute radius
	SumRadius=0;
	for (nPoint=0; nPoint<this->_mGetNPoints(); nPoint++)
		SumRadius+=this->_mComputeNorm(this->_vPoints[nPoint][0],
									   this->_vPoints[nPoint][1],
									   this->_vCenterX,
									   this->_vCenterY);
	this->_vRadius=SumRadius/((double)(this->_vNPoints));

	// -----------------------------------------
	// Return

	return Err;
}

// -----------------------------------------
// Compute baricenter
int	CircleFittingClass::_mComputeBaricenter()
{
	// -----------------------------------------
	// Variables

	int		Err=0;		// Error code
	double	SumX,SumY;	// SumX, SumY
	int		nPoint;		// nPoint index

	// -----------------------------------------
	// Compute baricenterusing namespace std;

	// Verify that the number of point is higher than 0
	if (this->_mGetNPoints()==0)
		return (Err=_CLASSCIRCLEFITTING_ERRORCOMPUTINGBARICENTER);

	// Compute baricenter
	SumX=SumY=0;
	for (nPoint=0; nPoint<this->_mGetNPoints(); nPoint++)
	{
		SumX+=this->_vPoints[nPoint][0];
		SumY+=this->_vPoints[nPoint][1];
	}
	this->_vCenterX=SumX/((double)(this->_mGetNPoints()));
	this->_vCenterY=SumY/((double)(this->_mGetNPoints()));

	// -----------------------------------------
	// Return

	return Err;
}

// -----------------------------------------
// Get center X
double	CircleFittingClass::mGetCenterX()
{
	return this->_vCenterX;
}

// -----------------------------------------
// Get center Y
double	CircleFittingClass::mGetCenterY()
{
	return this->_vCenterY;
}

// -----------------------------------------
// Get radius
double	CircleFittingClass::mGetRadius()
{
	return this->_vRadius;
}

double CircleFittingClass::mOMaMaCircleFitting_Computef(double CenterX, double CenterY, double Radius)
{
	// -----------------------------------------
	// Variables

	double	f;							// Function f value
	double	Phi;						// Phi
	double	SquaredRho;					// Squared rhos
	double	SquaredRadius;				// Squared radius
	int		nPoint;						// Point index

	// -----------------------------------------
	// Compute f

	// Sqaured radius
	SquaredRadius=Radius*Radius;

	// Init f
	Phi=this->_vTransition*SquaredRadius;
	if (Phi<=709.0)
		f=((double)(this->_mGetNPoints()))*log(log(1.0+exp(Phi)));
	else
		f=((double)(this->_mGetNPoints()))*log(Phi);
	
	// Compute f
	for (nPoint=0; nPoint<this->_mGetNPoints(); nPoint++)
	{
		// Sqaured rho
		SquaredRho=(this->_vPoints[nPoint][0]-CenterX)*(this->_vPoints[nPoint][0]-CenterX)+(this->_vPoints[nPoint][1]-CenterY)*(this->_vPoints[nPoint][1]-CenterY);

		// Phi
		Phi=this->_vTransition*(SquaredRho-SquaredRadius);

		// Adj f
		if (Phi<=709.0)
			f+=log(1.0+exp(Phi));
		else
			f+=Phi;
	}

	// -----------------------------------------
	// Return

	return f;
}

int CircleFittingClass::_mSimplexSum(double Simplex[4][3], double SimplexSum[3])
{
	int	nVertex,nDim;

	for (nDim=0; nDim<3; nDim++)
	{
		SimplexSum[nDim]=0.0;
		for (nVertex=0; nVertex<4; nVertex++)
			SimplexSum[nDim]+=Simplex[nVertex][nDim];
	}
	return 0;
}


int	CircleFittingClass::mOMaMaCircleFitting_DownhillSimplex(double Transition)
{
	// -----------------------------------------
	// Variables

	int		Err=0;				// Error code
	double	p[4][3];			// Simplex
	int		nVertex;			// Vertex index
	double	y[4];				// Function evaluation (negative log likelihood)

	// -----------------------------------------
	// Downhill simplex method

	// Set transition
	this->_vTransition=Transition;

	// Initialization
	for (nVertex=0; nVertex<4; nVertex++)
	{
		p[nVertex][0]=this->mGetCenterX()+this->mGetRadius()*(0.5-((double)(rand())/((double)(RAND_MAX))));
		p[nVertex][1]=this->mGetCenterY()+this->mGetRadius()*(0.5-((double)(rand())/((double)(RAND_MAX))));
		p[nVertex][2]=this->mGetRadius()+this->mGetRadius()*(0.5-((double)(rand())/((double)(RAND_MAX))));
		y[nVertex]=this->mOMaMaCircleFitting_Computef(p[nVertex][0],p[nVertex][1],p[nVertex][2]);
	}

	// Downhill simplex methos, rewritten from NR

	double  ftol=1.0e-4;

	const int NMAX=5000;
	const double TINY=1.0e-10;
	int i,ihi,ilo,inhi,j;
	double rtol,ysave,ytry;

	int mpts=4;
	int ndim=3;

	double psum[3];

	//Vec_DP psum(ndim);
	int nfunk=0;

	// Compute simplex sums
	this->_mSimplexSum(p,psum);

	// Simplex loop
	for (;;)
	{		
		//First we must determine which point is the highest (worst), next-highest, and lowest (best), by looping over the points in the simplex.
		ilo=0;
		ihi = y[0]>y[1] ? (inhi=1,0) : (inhi=0,1);
		for (i=0;i<mpts;i++)
		{
			if (y[i] <= y[ilo])
				ilo=i;
			if (y[i] > y[ihi])
			{
				inhi=ihi;
				ihi=i;
			}
			else if (y[i] > y[inhi] && i != ihi)
				inhi=i;
		}
		rtol=2.0*fabs(y[ihi]-y[ilo])/(fabs(y[ihi])+fabs(y[ilo])+TINY);

		// Compute the fractional range from highest to lowest and return if satisfactory.
		if (rtol < ftol)
		{
			// If returning, put best point and value in slot 0.
			//SWAP(y[0],y[ilo]);
			this->_mSwap(&(y[0]),&(y[ilo]));
			for (i=0;i<ndim;i++)
				//SWAP(p[0][i],p[ilo][i]);
				this->_mSwap(&(p[0][i]),&(p[ilo][i]));
			break;
		}
		if (nfunk >= NMAX)
			return (Err=_CLASSCIRCLEFITTING_ERRORSIMPLEXMAXITERREACHED);//nrerror("NMAX exceeded");
		nfunk += 2;

		// Begin a new iteration. First extrapolate by a factor -1 through the face of the simplex
		// across from the high point, i.e., reflect the simplex from the high point.
		//ytry=amotry(p,y,psum,funk,ihi,-1.0);
		ytry=this->_mAmoTry(p,y,psum,ihi,-1.0);
		if (ytry <= y[ilo])
			// Gives a result better than the best point, so try an additional extrapolation by a factor 2.
			//ytry=amotry(p,y,psum,funk,ihi,2.0);
			ytry=this->_mAmoTry(p,y,psum,ihi,2.0);
		else if (ytry >= y[inhi])
		{
			// The reflected point is worse than the second-highest, so look for an intermediate
			// lower point, i.e., do a one-dimensional contraction.
			ysave=y[ihi];
			//ytry=amotry(p,y,psum,funk,ihi,0.5);
			ytry=this->_mAmoTry(p,y,psum,ihi,0.5);
			if (ytry >= ysave) 
			{
				// Cant seem to get rid of that high point. Better
				// contract around the lowest (best) point.
				for (i=0;i<mpts;i++)
				{
					if (i != ilo)
					{
						for (j=0;j<ndim;j++)
							p[i][j]=psum[j]=0.5*(p[i][j]+p[ilo][j]);
						//y[i]=funk(psum);
						y[i]=this->mOMaMaCircleFitting_Computef(psum[0],psum[1],psum[2]);
					}
				}
				nfunk += ndim;		// Keep track of function evaluations.
				this->_mSimplexSum(p,psum);	// Recompute psum.
			}
			else --nfunk;
		}
	}

	// -----------------------------------------
	// Set output

	this->_vCenterX=p[0][0];
	this->_vCenterY=p[0][1];
	this->_vRadius=p[0][2];

	// -----------------------------------------
	// Return

	return Err;

}

double CircleFittingClass::_mAmoTry(double p[4][3], double y[4], double psum[4], int ihi, const double fac)
{
	int j;
	double fac1,fac2,ytry;

	//int ndim=p.ncols();
	int ndim=3;
	double	ptry[4];
	//Vec_DP ptry(ndim);
	fac1=(1.0-fac)/ndim;
	fac2=fac1-fac;
	for (j=0;j<ndim;j++)
		ptry[j]=psum[j]*fac1-p[ihi][j]*fac2;
	//ytry=funk(ptry,pData);
	ytry=this->mOMaMaCircleFitting_Computef(ptry[0],ptry[1],ptry[2]);
	if (ytry < y[ihi]) {
		y[ihi]=ytry;
		for (j=0;j<ndim;j++) {
			psum[j] += ptry[j]-p[ihi][j];
			p[ihi][j]=ptry[j];
		}
	}
	return ytry;
}

void CircleFittingClass::_mSwap(double *Value0, double *Value1)
{
	double Swap;
	Swap=*Value0;
	*Value0=*Value1;
	*Value1=Swap;
}


