#ifndef EventMgr_H_DEFINED
#define EventMgr_H_DEFINED
#include <OPENR/OObject.h>
#include <OPENR/OSubject.h>
#include <OPENR/OObserver.h>
#include <OPENR/RCRegion.h>
#include "def.h"
#include <queue>
#include "../headers/Geometry.h"
#include "../Motion/MotionInterface.h"
#include "../lib/AiboEdgeDetector/AiboEdgeDetector.h"
#include "../lib/AISColorSeg/AISAiboYCrCbColorSeg.h"
#include <OPENR/OFbkImage.h>
#include "../utils/utils.h"
#include<../lib/iuriFit/CircleFittingClass.h>
#include <math.h>
#include <../lib/matrix/matrix.h>


#ifndef _NO_NAMESPACE
using namespace std;
using namespace math;

#define STD std
#else
#define STD
#endif

#ifndef _NO_TEMPLATE
typedef matrix<double> Matrix;
#else
typedef matrix Matrix;
#endif



#define NUM_BALL_DISTANCE 10 //sempre numero pari maggiore di 2
#define NUM_BALL_LOCATION  NUM_BALL_DISTANCE
#define NUM_MEDIA_DIAMETRI 10
#define NUM_IMMAGINI_VELOCITA 5
#define NUM_IMMAGINI_CERCA_PALLA 45
#define NUM_BALL_DISTANCE_NOT_VALID_VALUE  -2147483 //valore di nn validita' delle x e y della palla nell'array delle locazioni della palla
#define CENTIMETRI_TUFFO_AL_CENTRO 100 //millimetri di distanza dalla verticale del cane per far s= che si tuffi al centro e nn a sx o dx, quindi la palla e' abbastanza vicina da tuffarsi al centro
#define SOGLIA_VEL_MAX_PALLA_FERMA 2 //soglia per stabilizzare la velocita', sotto  SOGLIA_VEL_MAX_PALLA_FERMA mm/frame la palla e' considerata ferma (2mm/frame = 6cm/s tantissimo!!)
#define LUNGHEZZA_TESTA 81.06
#define DISTANZA_PETTO_TESTA 80.0


/** Primitiva della telecamera.*/
static const char* const FBK_LOCATOR = "PRM:/r1/c1/c2/c3/i1-FbkImageSensor:F1";
/** Alcune primitive di sensori utili.*/
static const char* const SENSOR_LOCATOR[] = {
    "PRM:/r1/c1/c2-Joint2:12",   // HEAD PAN
    "PRM:/r1/c1/c2/c3-Joint2:13", // HEAD TILT2
    "PRM:/r1/c1-Joint2:11", // HEAD TILT
    "PRM:/r1/c1/c2/c3/p1-Sensor:p1",    // HEAD PSD (NEAR)
    "PRM:/r1/c1/c2/c3/p2-Sensor:p2"    // HEAD PSD (FAR)
};



class EventMgr : public OObject {
public:
  EventMgr();
  virtual ~EventMgr() {};
  OSubject*    subject[numOfSubject];
  OObserver*   observer[numOfObserver];
  virtual OStatus DoInit   (const OSystemEvent& event);
  virtual OStatus DoStart  (const OSystemEvent& event);
  virtual OStatus DoStop   (const OSystemEvent& event); 
  virtual OStatus DoDestroy(const OSystemEvent& event);
  void NotifyImage(const ONotifyEvent& event);
  void NotifySensor(const ONotifyEvent& event);
  void Ready(const OReadyEvent& event);
  void ReadyMotion(const OReadyEvent& event);
  void RLibInfoReady(const OReadyEvent& event);
  //void UpdateState();
  void TimerEnd(void* msg);
  static int GetFotoSync(void);
  static void SetFotoSync(int var);
  

private:
  
  static int fotoSync;
  queue<int> codaStati;
  int tmpstate;
  int state;
  int width;
  int height; 
  int timeoneshot;
  //creo gli oggetti per la segmentazione e per l'edge detection
 
  AISAiboYCrCbColorSeg* seg;
  AiboEdgeDetector* edge;
  int** lineeCampo;// [3][3] 3 dati per  ogni linea, theta, rho, qte in  immagini  stata trovata (realativa ad 1 scansione con la testa)
  int numLineeCampo;//  = 0; numero di linee del campo trovare
  int numImmaginiDaUltimoFrameProcessato;
  int numImmaginiCercaPalla;
  int numImmagini;
  int DelayAction;
  double prevFrameDistance;
  int prevState;
  double  *prevBallDistance; //array di  interi che contengono le distanze della palla nei  frame precedenti
  double  *prevBallDiameter; //array di  interi che contengono i diametri della palla nei  frame precedenti
  int clock;
  double prevTimeToImpact;
 
  struct info_Frame {
    double xcentroide;
    double ycentroide;
  };
  struct ballLocationStruct {
    int x;
    int y;
  };
  ballLocationStruct *ballLocation; //array contenente le ultime posizioni della palla rispetto al cane.
  RLibInfo res;

private:
    static const size_t      NUM_SENSOR_VECTOR  = 2;
    static const size_t      NUM_SENSORS        = 5;

    enum
      {
	PAN_INDEX= 0,
	TILT2_INDEX= 1,
	TILT_INDEX = 2,
	HEAD_IRSENSOR_NEAR=3,
	HEAD_IRSENSOR_FAR=4,
      };  

 
    OPrimitiveID fbkID;
    OPrimitiveID jointID[NUM_SENSORS];
    bool initSensorIndex;
    int sensoridx[NUM_SENSORS];
    list<RCRegion*> sensorRegions;
    double lastRefPan;
    double lastRefTilt;

private:
    //calcolo velocita' Media con la posizione della palla dei frame precedenti e l'attuale
    double calcoloVelMediaY();
    //calcolo velocita' Media con la posizione della palla dei frame precedenti e l'attuale
    double calcoloVelMediaX();
    /** Apre le primitive.*/
    void OpenPrimitives();
    /** Imposta i parametri della telecamera (gain, wb, e exposition).*/
    void SetCameraParameter();
    /** Inizializzo gli indici dei sensori.*/
    void InitSensorIndex(OSensorFrameVectorData* sensorVec);
    /** Ritorna gli angoli della testa.*/
    void GetPanTiltAngle(longword frameNum, double* pan, double* tilt,  double* tilt2);
    /** Ritorna le letture dai sensori di distanza.*/
    void GetIrHeadDistance(longword frameNum, int* near, int* far);
    /** Converte l'angolo di pan da gradi a ore (in stile militare).*/
    double pan2Hour(double pan) {return ((pan * -1) * 12) / 360;};
      /*calcola la trasformata di hough dell'immagine passata e ne ritorna la linea con maggiornnumero di pixel trovata */
    int** trovoLineaPrincipaleHough(const unsigned int* image,unsigned int numColore,int height,int width,  int sogliaEdge, int thetaInterval,int dInterval,const unsigned char* edgeImage);
    //le prossime 2 funzioni servono per salvare un frame della cam sulla scheda
    void SaveRawData(char* path,OFbkImageVectorData* imageVec, OFbkImageLayer layer);
    void SaveData(int serialNumber, OFbkImageVectorData* imageVec);
    int TimeToImpact(int* sxCentroDx,double* timeToImpact);//calcolo il tempo in secondi e lo metto in timetoimpact, e calcolo se cadr' a sx o destra e lo metto in sxdx.retituisco 0 se e' andato a buon fine
    double calcoloDiametroMedio(); //calcolo la distanza media della palla delle ultime NUM_MEDIA_DISTANZE
    double rotoTraslazioneVettorePalla(Matrix *vettorePalla, NeckAngle angoliAibo);//cambia gli assi di riferimento da quelli sul muso dell'aibo a quelli sul petto.questo per avere dei riferimenti fissi e poter calcolare precisamente la velocit
};

#endif
