#include "POView.h"
#include <OPENR/OSyslog.h>
#include <OPENR/core_macro.h>
#include <OPENR/OPENRAPI.h>
#include "entry.h"
#include "../Motion/MotionInterface.h"

POView::POView(){
  Motion::MotionCommand cmd;

  cmd.motion_cmd=Motion::MOTION_WALK_TROT;       
  cmd.head_cmd=Motion::HEAD_ANGLES;
  cmd.tail_cmd=Motion::TAIL_FOLLOW;
  cmd.bound_mode;       
  cmd.vx=150;
  cmd.vy=0;
  cmd.va=0;      
  cmd.head_lookat=vector3d(30,30,30);  
  cmd.head_tilt_offset=0;  
  cmd.head_tilt=-20.0; 
  cmd.head_pan=0;  
  cmd.head_roll=0; 
  cmd.head_tilt2=0; 
  cmd.tail_pan=60;       
  cmd.tail_tilt=0;   
  cmd.mouth=100;          
  cmd.sound_cmd=Motion::SOUND_FILE;
  cmd.sound_file=Motion::SOUND_FILE_RECTANGLE;
  cmd.sound_file=Motion::SOUND_FILE_TADA;

  cmd.setFaceLEDs(0,0,Motion::MODE_A);
  codaComandi.push(cmd);


}

OStatus POView::DoInit(const OSystemEvent& event){
  OSYSDEBUG(("POView::DoInit()\n"));
  
  NEW_ALL_SUBJECT_AND_OBSERVER;
  REGISTER_ALL_ENTRY;
  SET_ALL_READY_AND_NOTIFY_ENTRY;

  return oSUCCESS;
}


/*DoStart: fa partire Telepati*/
OStatus POView::DoStart(const OSystemEvent& event){
  OSYSDEBUG(("POView::DoStart()\n"));

  ENABLE_ALL_SUBJECT;
  ASSERT_READY_TO_ALL_OBSERVER;

  //OStatus st = OPENR::SetMotorPower(opowerON);   

  int dummy=0;
  EventID sincroEvent = EventID();
  RelativeTime period(10, 0);
  TimeEventInfoWithRelativeTime timeInfo(TimeEventInfo::Periodic, period);
  sError error=SetTimeEvent(&timeInfo,myOID_,Extra_Entry[entryTimerEnd],&dummy,sizeof(dummy),&sincroEvent);
  if(error!=sSUCCESS) 
    OSYSLOG1((osyslogERROR, "::DoStart() : ERROR: Sincronization Timer not started\n"));
  
  
  return oSUCCESS;
}  //DoStart() END


/*DoStop: arresta Telepati*/
OStatus POView::DoStop(const OSystemEvent& event){
  OSYSDEBUG(("POView::DoStop()\n"));

  DISABLE_ALL_SUBJECT;
  DEASSERT_READY_TO_ALL_OBSERVER;
  OStatus st = OPENR::SetMotorPower(opowerOFF); 

  return oSUCCESS;
}  //DoStop() END


/*DoDestroy: libera la memoria occupata da Telepati*/
OStatus POView::DoDestroy(const OSystemEvent& event){
  OSYSDEBUG(("POView::DoDestroy\()\n"));

  DELETE_ALL_SUBJECT_AND_OBSERVER;

  return oSUCCESS;
}  //DoDestroy() END

void POView::TimerEnd(void* msg){
  if(!codaComandi.empty())
    {
      subject[sbjMotionControl]->SetData(&(codaComandi.front()),sizeof(Motion::MotionCommand));
      subject[sbjMotionControl]->NotifyObservers();
      codaComandi.pop();
      
    }
}

void POView::Notify(const ONotifyEvent& event){
  OSYSDEBUG(("POView:: ricevuto un notify\n"));
}

void POView::Ready(const OReadyEvent& event){
  OSYSDEBUG(("POView:: ricevuto Assert Ready\n"));
}
