#include <iostream>
#include "AiboEdgeDetector.h"
#include <string>
#include "write_jpeg.h"
#include <sys/time.h>

#include <stdio.h>

using namespace std;

static const int gDefaultWidth=208;
static const int gDefaultHeight=160;

static const string FILE_NAME="image";

void PutYCbCrPixel( unsigned char* img, int w,
		    int x, int y,  char ypix,  char cb,  char cr) {
  unsigned char* ptr = img + (3 * (w * y + x));
  ptr[0] = ypix;
  ptr[1] = cb;
  ptr[2] = cr;
}

void ReconstructAndConvertCtd(const  unsigned char* imageVec, unsigned char* image,
			      int width, int height)
{
  const unsigned char* ptr=(imageVec);
  
  int yValue,crValue,cbValue;
  
  for (int y = 0; y < height; y++) 
    {
      for (int x = 0; x < width; x++) 
	{
	  yValue=(*ptr);
	  crValue=*ptr;
	  cbValue=*ptr;
	  
	  PutYCbCrPixel(image, width, x,   y,   yValue, 128, 128);
	  ptr++;
	}
    }
  return;
}

void ConvertYCbCr(const unsigned  char* imageVec,
		  unsigned char* image,
		  int width, int height)
{
  int counter = 0;
  int line = 0;
  
  image += 3;
  for(int x = 0; x < width*height ; x++){
    
    if(line == width-1){
      counter += 5*width+1; 
      line = 0;
    }
    else{
      counter++;
      line++;
    }
    
    *image++ = imageVec[counter]; //y
    *image++ = imageVec[counter+width*2]; //cb
    *image++ = imageVec[counter+width]; //cr
  }
}

/**
   Apre l'immagine raw ./image.raw quindi salva una copia in formato
   Jpeg dell'immagine originale (imageoriginal.jpg), salva una copia
   dell'immagine contenente il Magnitude presente nel formato raw
   (imageedge.jpg).
*/
int main(){
  struct timeval startTime, endTime;
  struct timezone zone= {0,0};
  
  AiboEdgeDetector segmentator(gDefaultWidth, gDefaultHeight);
  
  
  string rawImage="./" + FILE_NAME + ".raw";
  string originalImage="./" + FILE_NAME + "original.jpg";
  string segmentedImage="./" + FILE_NAME + "edge.jpg";
  
  unsigned char buffer[ (gDefaultWidth * gDefaultHeight) * 6 ];
  unsigned char recbuffer[ (gDefaultWidth * gDefaultHeight) * 3 ];
  unsigned char anotherbuffer[ (gDefaultWidth * gDefaultHeight) * 3 ];

    
  //
  // Apro l'immagine originale
  //
  FILE* tempFile;
  tempFile=fopen(rawImage.c_str(), "r");
  if(tempFile == NULL)
    {
      cerr << "Errore di apertuara file 1" << endl;
      return -1;
    }
  fread(buffer, 1,  (gDefaultWidth * gDefaultHeight) * 6 , tempFile);
  fclose(tempFile);

  ConvertYCbCr(buffer, recbuffer, gDefaultWidth, gDefaultHeight);
  
  //
  // Salvo l'immagine originale
  //
  cout << "Salvo l'immagine... ";
  tempFile=fopen(originalImage.c_str(), "w");
  if(tempFile == NULL)
    {
      cerr << "Errore di apertuara file 2" << endl;
      return -1;
    }
  write_jpeg_file((recbuffer), gDefaultWidth, gDefaultHeight, 95, tempFile);
  fclose(tempFile);

  cout << " ...salvata" << endl;
  
  cout << "Processo l'immagine... ";
  //
  // Processo l'immagine
  //
  gettimeofday(&startTime, &zone);
  const unsigned char* processedImg=segmentator.getMagnitude(buffer);
  gettimeofday(&endTime, &zone);
  cout << " ...processata" << endl;
  
  cout << "Ci ho messo: " << endTime.tv_sec-startTime.tv_sec << "sec " << endTime.tv_usec - startTime.tv_usec<< "usec" << endl;

  cout << "Salvo l'immagine processata..." ;

  
  //
  // Salvo l'immagine processata
  //
  ReconstructAndConvertCtd(processedImg, anotherbuffer, gDefaultWidth, gDefaultHeight);
  tempFile=fopen(segmentedImage.c_str(), "w");
  if(tempFile == NULL)
    {
      cerr << "Errore di apertuara file 2" << endl;
      return -1;
    }
  
  write_jpeg_file(anotherbuffer, gDefaultWidth, gDefaultHeight, 95, tempFile);

  fclose(tempFile);
  cout << " ...salvata" << endl;
  
  
  return 0;
}
