/****************************************************************************
 * This file was derived from the CNCL library by the Pablo research group.
 *
 * The complete CNCL library is available at:
 *       http://www.comnets.rwth-aachen.de/doc/cncl.html
 *
 * The CNCL copyright notice is included here:
 *
 * |_|_|_  |_|_    |_    |_|_|_  |_               C O M M U N I C A T I O N
 *|_       |_  |_  |_  |_        |_                         N E T W O R K S
 *|_       |_  |_  |_  |_        |_                               C L A S S
 * |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_                     L I B R A R Y
 *
 *
 *
 * Copyright (C) 1992-1998   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@comnets.rwth-aachen.de
 *
 *
 * This library is free  software; you can  redistribute it and/or modify
 * it  under   the terms  of the GNU   Library  General Public License as
 * published by the  Free Software Foundation;  either  version 2 of  the
 * License, or  (at  your option)  any  later  version.  This  library is
 * distributed  in the  hope  that  it  will  be useful,  but WITHOUT ANY
 * WARRANTY;  without even the   implied  warranty of MERCHANTABILITY  or
 * FITNESS FOR A PARTICULAR PURPOSE.   See the GNU Library General Public
 * Library General Public License along with  this library; if not, write
 * to the Free Software Foundation,  675  Mass Ave, Cambridge, MA  02139,
 * USA.
 ****************************************************************************/

//   -*- C++ -*-
/*****************************************************************************
 *
 * Class: FlFNumTriangle --- Fuzzy number (triangle)
 *
 *****************************************************************************/

#ifndef __FlFNumTriangle_h
#define __FlFNumTriangle_h


#include <FlFSetTriangle.h>		// Base class

/*
 * The class FlFNumTriangle
 */

class FlFNumTriangle : public FlFSetTriangle
{
  public:	/***** Constructors ******************************************/
    FlFNumTriangle()                    : FlFSetTriangle(0, 0, 0) {}
    FlFNumTriangle(double m, double a, double b) : FlFSetTriangle(m, a, b) {}
    FlFNumTriangle(const FlFNumTriangle &n);
    
  public:	/***** Public interface **************************************/
    virtual double center_of_gravity(double min, double max) const;
    double defuzzy() const;

    FlFNumTriangle operator - ();
    FlFNumTriangle operator + ();

    FlFNumTriangle & operator = (const FlFNumTriangle &n);

    friend FlFNumTriangle operator + (FlFNumTriangle, FlFNumTriangle);
    friend FlFNumTriangle operator - (FlFNumTriangle, FlFNumTriangle);
    friend FlFNumTriangle operator * (double, FlFNumTriangle);
    friend FlFNumTriangle operator * (FlFNumTriangle, double);

    friend FlFNumTriangle sqr(FlFNumTriangle);
    friend FlFNumTriangle abs(FlFNumTriangle);

    // Print/debug output
    virtual void print(ostream &strm = cout) const;
    virtual void dump (ostream &strm = cout) const;
};

#endif /**__FlFNumTriangle_h**/
