/****************************************************************************
 * This file was derived from the CNCL library by the Pablo research group.
 *
 * The complete CNCL library is available at:
 *       http://www.comnets.rwth-aachen.de/doc/cncl.html
 *
 * The CNCL copyright notice is included here:
 *
 * |_|_|_  |_|_    |_    |_|_|_  |_               C O M M U N I C A T I O N
 *|_       |_  |_  |_  |_        |_                         N E T W O R K S
 *|_       |_  |_  |_  |_        |_                               C L A S S
 * |_|_|_  |_    |_|_    |_|_|_  |_|_|_|_                     L I B R A R Y
 *
 *
 *
 * Copyright (C) 1992-1998   Communication Networks
 *                           Aachen University of Technology
 *                           D-52056 Aachen
 *                           Germany
 *                           Email: cncl-adm@comnets.rwth-aachen.de
 *
 *
 * This library is free  software; you can  redistribute it and/or modify
 * it  under   the terms  of the GNU   Library  General Public License as
 * published by the  Free Software Foundation;  either  version 2 of  the
 * License, or  (at  your option)  any  later  version.  This  library is
 * distributed  in the  hope  that  it  will  be useful,  but WITHOUT ANY
 * WARRANTY;  without even the   implied  warranty of MERCHANTABILITY  or
 * FITNESS FOR A PARTICULAR PURPOSE.   See the GNU Library General Public
 * Library General Public License along with  this library; if not, write
 * to the Free Software Foundation,  675  Mass Ave, Cambridge, MA  02139,
 * USA.
 ****************************************************************************/

//   -*- C++ -*-
/*****************************************************************************
 *
 * Class: FlFSetTriangle --- Fuzzy set mit triangle function
 *
 *****************************************************************************/

#ifndef __FlFSetTriangle_h
#define __FlFSetTriangle_h

#include <FlFSetTrapez.h>		// Base class

/*
 * The class FlFSetTriangle
 */

class FlFSetTriangle : public FlFSetTrapez
{
  public:
					// Get membership values for x
    					// Optimized version for triangle
    virtual double get_membership(double x) const;

    double get_mean() const { return m1; }

    /***** Constructors ******************************************************/
  public:
    FlFSetTriangle();			// Default constructor

    FlFSetTriangle(double min, double max, 
		   double xm, double xalpha, double xbeta );
    FlFSetTriangle(double xm, double xalpha, double xbeta );
    FlFSetTriangle(FlStringR xname, double min, double max, 
		   double xm, double xalpha, double xbeta );
    FlFSetTriangle(FlStringR xname,
		   double xm, double xalpha, double xbeta );

    void init(FlStringR xname, 
	      double xm, double xalpha, double xbeta,
	      double min=FSET_MIN, double max=FSET_MAX );

    // Print/debug output
    virtual void print(ostream &strm = cout) const;
    virtual void dump (ostream &strm = cout) const;
};

#endif /**__FlFSetTriangle_h**/
